import os
import sys
import collections
#import ConfigParser
import configparser
import shutil
import argparse
import math
import numpy
import time
import string
from xml.etree import ElementTree
from xml.dom import minidom

import structure.crystal
import structure.atom
#import calculator
#import vasp
#import abinit
#import espresso
#import calprop
import calculation.calculator
import calculation.vasp
#from calculation.vasp import *
import calculation.abinit
import calculation.espresso
import calculation.calprop
import jobmanage.job
#import toast_config
import calculation.toast_config


class Task():
    def __init__(self, name = None):
        self.name = name
        self.input = collections.OrderedDict()
        self.option = collections.OrderedDict()

class Calc():
    version = "0.31.037"
    program = None
    # version = None
    pplist = None
    matid = ""
    nproc = 0
    scenario = None
    spin = None
    spin_criterion = 0.1
    kpoints_length = 30
    kpoints_spacing = None
    kpoints_max_mesh = None
    kpoints_num_path = 100
    kpoints_mesh_nk1 = 0
    kpoints_mesh_nk2 = 0
    kpoints_mesh_nk3 = 0
    cif = None
    fout = "calculations.xml"
    fmdat = "metadata.xml"

    ele = None
    convcell = None
    primcell = None
    recipcell = None
    atom_conv = None
    atom_prim = None

    def __init__(self, program, matid, nproc = 0, scenario = "default"):
        self.program = program.lower()
        self.matid = matid
        self.nproc = nproc
        self.scenario = scenario

        self.program_options = {}
        self.job_options = {}

        #toast_cfg_file = os.path.join(toast_config.cfgdir, "toast.cfg")
        toast_cfg_file = os.path.join(calculation.toast_config.cfgdir, "toast.cfg")
        if not os.path.isfile(toast_cfg_file):
            sys.stderr.write("Error can not find {}".format(toast_cfg_file))
            exit(1)
        #toast_cfg = ConfigParser.SafeConfigParser()
        toast_cfg = configparser.SafeConfigParser()
        toast_cfg.read(toast_cfg_file)

        if toast_cfg.has_section(self.program):
            self.program_options.update(toast_cfg.items(self.program))
        if toast_cfg.has_section("job"):
            self.job_options.update(toast_cfg.items("job"))

        if self.program=="vasp":
            #self.cal = vasp.Vasp()
            self.cal = calculation.vasp.Vasp()
        elif self.program=="abinit":
            #self.cal = abinit.Abinit()
            self.cal = calculation.abinit.Abinit()
        elif self.program=="espresso":
            #self.cal = espresso.Espresso()
            self.cal = calculation.espresso.Espresso()

        self.default_input = collections.OrderedDict()
        self.default_option = collections.OrderedDict()
        self.task_dic = collections.OrderedDict()

    def calculate(self):
        print("calc...",self.matid)

        # scenario
        if not os.path.exists(self.scenario):
            #s = os.path.join(toast_config.cfgdir, self.scenario)
            s = os.path.join(calculation.toast_config.cfgdir, self.scenario)
            if os.path.exists(s):
                self.scenario = s
            elif self.scenario == "default":
                #self.scenario = os.path.join(toast_config.cfgdir, self.program_options["scenario"])
                self.scenario = os.path.join(calculation.toast_config.cfgdir, self.program_options["scenario"])
            else:
                print("scenario not found")
                sys.exit(1)
        self.scenario = os.path.abspath(self.scenario)
        print("[scenario]\n",self.scenario)

        # kpoint max mesh (default)
        self.kpoints_max_mesh = self.cal.nkp_max

        self.read_config(self.scenario)

        self.cal.matid = self.matid

        self.cal.ppdir = self.program_options["ppdir"]
        self.cal.pplist = self.program_options["pplist"]
        self.cal.cmd = self.program_options["cmd"]      # XXX work around for abinit.charge

        # structure
        print("[structure]")
        self.structure()

        # preparation
        print("[prep]")
        self.calc_prep()
        self.write_parameters(self.fout)
        # temporary
        self.write_metadata(self.fmdat)

        # calculation
        for name, task in self.task_dic.items():
            if name=="fermisurf" or name=="emass":
                continue
            print("["+name+"]")
            self.calc_task(task)

        # post
        print("[post]")
        self.calc_post()

        bandgap_type = self.bandgaptype(self.fout)
        #print("bandgap_type:",bandgap_type)
        if "fermisurf" in self.task_dic:
            if bandgap_type == "metal":
                print("[fermisurf]")
                self.calc_task(self.task_dic["fermisurf"])
                #calprop.fermisurf()
                calculation.calprop.fermisurf()
        if "emass" in self.task_dic:
            if bandgap_type != "metal":
                print("[emass]")
                self.calc_task(self.task_dic["emass"])
                #calprop.effectivemass()
                calculation.calprop.effectivemass()

        # temporary
        os.system("xmllint --format "+self.fout+" > tmp1.xml")
        os.system("mv tmp1.xml "+self.fout)
        os.system("xmllint --format "+self.fmdat+" > tmp2.xml")
        os.system("mv tmp2.xml "+self.fmdat)

    def read_config(self,f):
        # self.task_dic = collections.OrderedDict()

        print("read config...",os.path.basename(f))
        #doc = ConfigParser.SafeConfigParser(allow_no_value=True)
        doc = configparser.SafeConfigParser(allow_no_value=True)
        doc.read(f)
        for s in doc.sections():
            #print("[%s]" % s)
            inp = collections.OrderedDict()
            cal = collections.OrderedDict()
            for k,v in doc.items(s):
                v = doc.get(s,k)
                #print("%s = %s" % (k,v))
                if k[0:1] == "_" and k.find("_namelist")<0:
                    cal[k] = v
                else:
                    inp[k] = v
            if s == "general":
                # skip general section for compatibility
                continue
            elif s == "default":
                self.default_input = inp.copy()
                self.default_option = cal.copy()
                # spin
                if "_spin" in cal:
                    if cal["_spin"] == "nomag":
                        self.spin = "nomag"
                    elif cal["_spin"] == "ferro":
                        self.spin = "ferro"
                # kpoint
                if "_kpoints_length" in cal:
                    self.kpoints_length = cal["_kpoints_length"]
                if "_kpoints_spacing" in cal:
                    self.kpoints_spacing = cal["_kpoints_spacing"]
                if "_kpoints_max_mesh" in cal:
                    self.kpoints_max_mesh = cal["_kpoints_max_mesh"]
                if "_kpoints_num_path" in cal:
                    self.kpoints_num_path = cal["_kpoints_num_path"]
                if "_kpoints_mesh_nk1" in cal:
                    self.kpoints_mesh_nk1 = cal["_kpoints_mesh_nk1"]
                if "_kpoints_mesh_nk2" in cal:
                    self.kpoints_mesh_nk2 = cal["_kpoints_mesh_nk2"]
                if "_kpoints_mesh_nk3" in cal:
                    self.kpoints_mesh_nk3 = cal["_kpoints_mesh_nk3"]
            else:
                self.task_dic[s] = Task(s)
                self.task_dic[s].option = self.default_option.copy()
                self.task_dic[s].option.update(cal.copy())
                foo = self.task_dic[s].option.get("_input", None)
                if foo == "default":
                    self.task_dic[s].input = self.default_input.copy()
                self.task_dic[s].input.update(inp.copy())

                if s=="check_spin" and "_spin_criterion" in cal:
                    self.spin_criterion = cal["_spin_criterion"]

        print("calculation (task)...")
        for s in self.task_dic.keys():
            print("["+s+"]",)
        print()

    def structure(self):
        cif = self.matid + ".cif"
        if not os.path.isfile(cif):
            cif = self.matid + "-1-2.cif"
        self.cif = cif
        print("cif file:",self.cif)

        cry = structure.crystal.Crystal()
        cry.read_cif(cif)
        cry.check_cif()
        cry.construct_structure()
        kpath = cry.get_kpoints(int(self.kpoints_num_path))
        cry.write_brillouinzone_gnuplot(self.matid+".bz.plt")
        cry.write_brillouinzone_jmol(self.matid+".bz.spt")
        cry.print_log()

        chemform = cry.chemical_formula_sum
        crysys = cry.space_group_crystal_system
        spgname = cry.symmetry_space_group_name
        spgnum = cry.symmetry_Int_Tables_number
        cell_a = cry.cell.length_a
        cell_b = cry.cell.length_b
        cell_c = cry.cell.length_c
        cell_alpha = cry.cell.angle_alpha
        cell_beta = cry.cell.angle_beta
        cell_gamma = cry.cell.angle_gamma
        primcell = cry.unit_prim
        convcell = cry.unit_conv
        bravais_center = cry.bravais.center
        bravais_subtype = cry.bravais.subtype

        ele = []
        for e in range(len(cry.velement_name)):
            ele.append(cry.velement_name[e])
        natom_ele = []
        for e in range(len(cry.velement_size)):
            natom_ele.append(cry.velement_size[e])
        #print("element:",ele)
        #print("natom/ele":,natom_ele)

        print(ele,natom_ele)
        if not chemform:
            s = [e+str(na) for e,na in zip(ele,natom_ele)]
            #chemform = string.join(s," ")
            chemform = " ".join(s)
        #print("chemical_fomula:",chemical_formula)

        atom_conv = []  # ele,pos,site,op
        for a in cry.vatom_conv:
            atom_conv.append([a.element,a.position[0],a.position[1],a.position[2],a.site_index,a.operation_index])
        atom_prim = []  # ele,pos,site,op
        for a in cry.vatom_prim:
            atom_prim.append([a.element,a.position[0],a.position[1],a.position[2],a.site_index,a.operation_index])

        atom = []
        for a in cry.vatom_prim:
            atom.append([a.element,a.position[0],a.position[1],a.position[2]])
        nele = len(ele)
        natom = len(atom)
        #print(nele, natom)

        self.ele = ele
        self.natom_ele = natom_ele
        self.convcell = convcell
        self.primcell = primcell
        self.atom_conv = atom_conv
        self.atom_prim = atom_prim
        la,lb,lc = celllength(primcell)
        aa,ab,ac = cellangle(primcell)
        #print("cell_length:",la,lb,lc)
        #print("cell_angle:",aa,ab,ac)
        volume = cellvolume(primcell)
        #print("volume:",volume)
        recipcell = cellrecip(primcell)
        #print("recipcell:",recipcell)
        self.recipcell = recipcell

        self.cal.chemical_formula = chemform
        self.cal.cry_system = crysys
        self.cal.spg_name = spgname
        self.cal.spg_num = spgnum
        self.cal.cell_a = cell_a
        self.cal.cell_b = cell_b
        self.cal.cell_c = cell_c
        self.cal.cell_alpha = cell_alpha
        self.cal.cell_beta = cell_beta
        self.cal.cell_gamma = cell_gamma
        #self.cal.primcell = primcell
        #self.cal.convcell = convcell
        self.cal.cell = primcell
        self.cal.rcell = recipcell
        self.cal.ele = ele
        self.cal.atom = atom
        self.cal.nele = nele
        self.cal.natom = natom
        self.cal.natom_ele = natom_ele
        self.cal.kpath = kpath
        self.cal.bravais_center = bravais_center
        self.cal.bravais_subtype = bravais_subtype

    def calc_prep(self):
        # element
        self.cal.elements = {}
        #f = open(toast_config.cfgdir+"/elements.aw.txt")
        f = open(calculation.toast_config.cfgdir+"/elements.aw.txt")
        for l in f:
            if "#" in l:
                continue
            l0 = l.split()
            atomno = l0[0]
            e = l0[1]
            w = l0[2]
            self.cal.elements[e] = [atomno,w]

        # pplist
        self.cal.pp = collections.OrderedDict()
        #f = open(toast_config.cfgdir+"/"+self.cal.pplist)
        f = open(calculation.toast_config.cfgdir+"/"+self.cal.pplist)
        for l in f:
            if "#" in l:
                continue
            l0 = l.split()
            e = l0[1]
            pp = l0[2]
            #print("pp:",pp)
            self.cal.pp[e] = pp
        f.close()

        self.cal.potentials(self.ele)
        self.nbands = self.cal.nbands(self.ele,self.natom_ele)
        self.cal.nbands = self.nbands
        self.cal.nkp = self.kpoints()
        if self.kpoints_max_mesh != None:
            self.cal.nkp_max = int(self.kpoints_max_mesh)
        print("kpoints:",self.cal.nkp[0],self.cal.nkp[1],self.cal.nkp[2])

    def write_parameters(self,f):
        root = ElementTree.Element("calculations")

        e0 = ElementTree.SubElement(root,"date")
        e0.text = str(time.ctime())
        # program
        e0 = ElementTree.SubElement(root,"program")
        e1 = ElementTree.SubElement(e0,"name")
        e1 = ElementTree.SubElement(e0,"version")
        # material
        e0 = ElementTree.SubElement(root,"material")
        e1 = ElementTree.SubElement(e0,"material_id")
        e1.text = self.matid
        e1 = ElementTree.SubElement(e0,"chemical_formula")
        e1.text = self.cal.chemical_formula
        e1 = ElementTree.SubElement(e0,"crystal_system")
        e1.text = self.cal.cry_system
        e1 = ElementTree.SubElement(e0,"spacegroup")
        e1.text = self.cal.spg_name
        e1 = ElementTree.SubElement(e0,"spacegroup_no")
        e1.text = str(self.cal.spg_num)
        e1 = ElementTree.SubElement(e0,"cif")
        e1.text = str(self.cif)

        # structure
        # conventional cell
        e0 = ElementTree.SubElement(root,"structure")
        e0.set("name","original")
        e1 = ElementTree.SubElement(e0,"lattice")
        e1.set("type","conventional")
        e2 = ElementTree.SubElement(e1,"a")
        e2.text = str(self.cal.cell_a)
        e2 = ElementTree.SubElement(e1,"b")
        e2.text = str(self.cal.cell_b)
        e2 = ElementTree.SubElement(e1,"c")
        e2.text = str(self.cal.cell_c)
        e2 = ElementTree.SubElement(e1,"alpha")
        e2.text = str(self.cal.cell_alpha)
        e2 = ElementTree.SubElement(e1,"beta")
        e2.text = str(self.cal.cell_beta)
        e2 = ElementTree.SubElement(e1,"gamma")
        e2.text = str(self.cal.cell_gamma)
        e1 = ElementTree.SubElement(e0,"lattice")
        e1.set("type","conventional")
        e2 = ElementTree.SubElement(e1,"a1")
        e2.text = str(self.convcell[0][0])+" "+str(self.convcell[0][1])+" "+str(self.convcell[0][2])
        e2 = ElementTree.SubElement(e1,"a2")
        e2.text = str(self.convcell[1][0])+" "+str(self.convcell[1][1])+" "+str(self.convcell[1][2])
        e2 = ElementTree.SubElement(e1,"a3")
        e2.text = str(self.convcell[2][0])+" "+str(self.convcell[2][1])+" "+str(self.convcell[2][2])
        # atom
        e1 = ElementTree.SubElement(e0,"atom")
        e1.set("type","conventional")
        for a in self.atom_conv:
            #s = ("%10.5f %10.5f %10.5f" % (a[1],a[2],a[3]))
            s = ("%15.10f %15.10f %15.10f" % (a[1],a[2],a[3]))
            e2 = ElementTree.SubElement(e1,"i")
            e2.set("name",a[0])
            e2.set("site",str(a[4]))
            e2.set("operation",str(a[5]))
            e2.text = s.strip()

        # primitive cell
        e1 = ElementTree.SubElement(e0,"lattice")
        e1.set("type","primitive")
        e2 = ElementTree.SubElement(e1,"a1")
        e2.text = str(self.cal.cell[0][0])+" "+str(self.cal.cell[0][1])+" "+str(self.cal.cell[0][2])
        e2 = ElementTree.SubElement(e1,"a2")
        e2.text = str(self.cal.cell[1][0])+" "+str(self.cal.cell[1][1])+" "+str(self.cal.cell[1][2])
        e2 = ElementTree.SubElement(e1,"a3")
        e2.text = str(self.cal.cell[2][0])+" "+str(self.cal.cell[2][1])+" "+str(self.cal.cell[2][2])
        # atom
        e1 = ElementTree.SubElement(e0,"atom")
        e1.set("type","primitive")
        for a in self.atom_prim:
            #s = ("%10.5f %10.5f %10.5f" % (a[1],a[2],a[3]))
            s = ("%15.10f %15.10f %15.10f" % (a[1],a[2],a[3]))
            e2 = ElementTree.SubElement(e1,"i")
            e2.set("name",a[0])
            e2.set("site",str(a[4]))
            e2.set("operation",str(a[5]))
            e2.text = s.strip()

        # reciprocal cell
        e1 = ElementTree.SubElement(e0,"lattice")
        e1.set("type","reciprocal")
        e2 = ElementTree.SubElement(e1,"a1")
        e2.text = str(self.cal.rcell[0][0])+" "+str(self.cal.rcell[0][1])+" "+str(self.cal.rcell[0][2])
        e2 = ElementTree.SubElement(e1,"a2")
        e2.text = str(self.cal.rcell[1][0])+" "+str(self.cal.rcell[1][1])+" "+str(self.cal.rcell[1][2])
        e2 = ElementTree.SubElement(e1,"a3")
        e2.text = str(self.cal.rcell[2][0])+" "+str(self.cal.rcell[2][1])+" "+str(self.cal.rcell[2][2])

        e0 = ElementTree.SubElement(root,"structure")
        e0.set("name","relaxed")

        tree = ElementTree.tostring(root,'utf-8')
        f0 = open(f,"w")
        #minidom.parseString(tree).writexml(f0,indent='',addindent='  ',newl='\n',encoding="utf-8")
        minidom.parseString(tree).writexml(f0)
        f0.close()

    def write_parameters_calc(self, task, f):
        name = task.name
        doc = ElementTree.parse(f)
        root = doc.getroot()

        # calculation
        e0 = ElementTree.SubElement(root,"calculation")
        e0.set("name",name)
        d = task.option["_calc_dir"]
        e0.set("dir",d)
        # inputs
        e1 = ElementTree.SubElement(e0,"inputs")
        #for k,v in task.option.iteritems():
        for k,v in task.option.items():
            e2 = ElementTree.SubElement(e1,"i")
            e2.set("name",k)
            e2.text = str(v).strip()
        #for k,v in task.input.iteritems():
        for k,v in task.input.items():
            e2 = ElementTree.SubElement(e1,"i")
            e2.set("name",k)
            e2.text = str(v).strip()

        # structure
        e1 = ElementTree.SubElement(e0,"structure")
        e1.set("name","initial")
        e1 = ElementTree.SubElement(e0,"structure")
        e1.set("name","final")

        # pseudopotential
        e1 = ElementTree.SubElement(e0,"pseudopotentials")
        for e in self.ele:
            pp = self.cal.pp[e]
            e2 = ElementTree.SubElement(e1,"i")
            e2.set("name",e)
            e2.text = pp

        # kpoints
        e1 = ElementTree.SubElement(e0,"kpoints")
        if name == "band" or name == "emass":
            e1.set("type","kpath")
            for k in self.cal.kpath:
                e2 = ElementTree.SubElement(e1,"i")
                if len(k)>4:
                    e2.set("name",str(k[4]))
                #e2.text = str(k[0])+" "+str(k[1])+" "+str(k[2])
                #s = "%10.5f %10.5f %10.5f" % (k[0],k[1],k[2])
                s = "%15.10f %15.10f %15.10f" % (k[0],k[1],k[2])
                e2.text = s.strip()
        else:
            e1.set("type","kmesh")
            s = ("%d %d %d" % (self.cal.nkp[0],self.cal.nkp[1],self.cal.nkp[2]))
            e1.text = s

        # parameters
        e1 = ElementTree.SubElement(e0,"parameters")

        # property
        e1 = ElementTree.SubElement(e0,"properties")

        # step
        e1 = ElementTree.SubElement(e0,"step")

        tree = ElementTree.tostring(root,'utf-8')
        f0 = open(f,"w")
        #minidom.parseString(tree).writexml(f0,indent='',addindent='  ',newl='\n',encoding="utf-8")
        minidom.parseString(tree).writexml(f0)
        f0.close()

    def write_metadata(self,f):
        root = ElementTree.Element("rdf:RDF")

        # rdf
        root.set("xmlns:rdf","http://www.w3.org/1999/02/22-rdf-syntax-ns#")
        root.set("xmlns:dc","http://purl.org/dc/elements/1.1/")

        e0 = ElementTree.SubElement(root,"rdf:Description")
        e0.set("rdf:about","http://mi2i.nims.go.jp/")

        e1 = ElementTree.SubElement(e0,"dc:title")
        e1.text = self.matid
        e1 = ElementTree.SubElement(e0,"dc:subject")
        e1.text = "Computational Electronic Structure"
        e1 = ElementTree.SubElement(e0,"dc:description")
        e1.text = self.cal.chemical_formula
        e1 = ElementTree.SubElement(e0,"dc:description")
        e1.text = self.cal.cry_system+" "+self.cal.spg_name+" "+str(self.cal.spg_num)
        e1 = ElementTree.SubElement(e0,"dc:description")
        e1.text = "Computational Electronic Structure"
        e1 = ElementTree.SubElement(e0,"dc:description")
        e1.text = "First-principles Calculations"
        e1 = ElementTree.SubElement(e0,"dc:type")

        cry = structure.crystal.Crystal()
        cry.read_cif(self.cif)
        pubname = cry.journal_name_full
        pubyear = cry.journal_year
        pubvol = cry.journal_volume
        pubpage1 = cry.journal_page_first
        pubpage2 = cry.journal_page_last
        e1 = ElementTree.SubElement(e0,"dc:relation")
        e1.text = pubname+" "+pubvol+" "+pubpage1+"-"+pubpage2+" "+pubyear
        e1 = ElementTree.SubElement(e0,"dc:creator")
        e1 = ElementTree.SubElement(e0,"dc:date")
        e1.text = str(time.ctime())

        tree = ElementTree.tostring(root,'utf-8')
        f0 = open(f,"w")
        #minidom.parseString(tree).writexml(f0,indent='',addindent='  ',newl='\n',encoding="utf-8")
        minidom.parseString(tree).writexml(f0)
        f0.close()

    def calc_task(self, task):
        caldir = os.getcwd()
        #print(caldir)

        if "_calc_dir" in task.option:
            d = task.option["_calc_dir"]
            if not os.path.exists(d):
                os.mkdir(d)
            os.chdir(d)

        #for k,v in task.option.iteritems():
        for k,v in task.option.items():
            if k[0:6] == "_mkdir":
                f = v
                print("mkdir",f)
                if not os.path.exists(f):
                    os.mkdir(f)
            if k[0:5] == "_copy":
                f = v.split()
                print("copy",f[0],f[1])
                if os.path.exists(f[0]):
                    shutil.copy2(f[0],f[1])

        # structure
        if "_structure" in task.option:
            print("structure:",task.option["_structure"])
            prev = task.option["_structure"]
            if prev != "cif":
                dir = self.task_dic[prev].option["_calc_dir"]
            else:
                dir = None
            self.cal.structure(prev, dir)

        # input
        nkp = self.cal.nkp
        #print("kpoints:",nkp[0],nkp[1],nkp[2])
        #print("nbands:",self.cal.nbands)
        if task.name == "check_spin":
            self.cal.spin=2
        elif self.spin == "nomag":
            self.cal.spin=1
        elif self.spin == "ferro":
            self.cal.spin=2
        elif "check_spin" in self.task_dic:
            d = self.task_dic["check_spin"].option["_calc_dir"]
            mag = self.cal.mag(d)
            self.cal.spin = self.ispin(mag)
        else:
            self.cal.spin = 1
        print("spin:",self.cal.spin)

        if task.name == "dos" or task.name == "tdos" or task.name == "pdos":
            d = self.task_dic["scf"].option["_calc_dir"]
            efermi = self.cal.efermi(d)
            emin = int(efermi-20);
            #emin = int(efermi-30);
            emax = int(efermi+20);
            self.cal.emin=emin
            self.cal.emax=emax
        if task.name == "band":
            #self.cal.kpath = self.kpath
            self.cal.kpath = self.kpoints_band()
        if task.name == "emass":
            self.cal.kpath = self.kpoints_emass(caldir+"/"+self.fout)
        self.cal.inputs(task)

        # make jobscript
        mpicmd = self.job_options.get("mpi_cmd", "mpirun -np")
        cmd = self.program_options["cmd"]
        if "_cmd" in task.option:
            cmd = task.option["_cmd"]
        if "bindir" in self.program_options:
            cmd = os.path.join(self.program_options["bindir"], cmd)
        #if self.nproc > 0:
        if int(self.nproc) > 0:
            nproc = self.nproc
        elif int(self.job_options["nproc"]) > 0:
            nproc = self.job_options["nproc"]
        else:
            nproc = "1"

        jobmanage.job.jobsh(cmd, mpicmd, nproc, task.option)

        # parameters
        self.write_parameters_calc(task, caldir+"/"+self.fout)

        # execute
        self.execute(task)

        # results
        self.cal.results(task, caldir+"/"+self.fout)

        if "_calc_dir" in task.option:
            os.chdir("..")

    def calc_post(self):
        doc = ElementTree.parse("calculations.xml")
        root = doc.getroot()
        
        #calprop.cellparameter(root)
        #calprop.bandgap(root)
        calculation.calprop.cellparameter(root)
        calculation.calprop.bandgap(root)

        tree = ElementTree.tostring(root,'utf-8')
        f0 = open("calculations.xml","w")
        #minidom.parseString(tree).writexml(f0,indent='',addindent='  ',newl='\n',encoding="utf-8")
        minidom.parseString(tree).writexml(f0)
        f0.close()

        #calprop.dosplot()
        #calprop.pdosplot()
        #calprop.bandplot()
        #calprop.scplot()
        #calprop.caldata()
        calculation.calprop.dosplot()
        calculation.calprop.pdosplot()
        calculation.calprop.bandplot()
        calculation.calprop.scplot()
        calculation.calprop.caldata()

        # charge cube
        d = self.task_dic["scf"].option["_calc_dir"]
        shutil.copy2(d+"/charge.cube",self.matid+".charge.cube")

        # cif (p1)
        cry = structure.crystal.Crystal()
        #cry.read_structure_vasp("POSCAR"):
        cry.unit_prim[0] = numpy.array(self.cal.cell[0],float)
        cry.unit_prim[1] = numpy.array(self.cal.cell[1],float)
        cry.unit_prim[2] = numpy.array(self.cal.cell[2],float)
        cry.velement_name = []
        for e in self.cal.ele:
            cry.velement_name.append(e)
        cry.velement_size = []
        for e in self.cal.natom_ele:
            cry.velement_size.append(int(e))
        a = self.cal.atom
        ii=0
        for i in range(len(cry.velement_name)):
            for j in range(cry.velement_size[i]):
                a0 = structure.atom.Atom()
                a0.position = numpy.array([a[ii][1],a[ii][2],a[ii][3]],float)
                a0.label = "%s%d" % (a[ii][0], j+1)
                a0.occupancy = 1.0
                a0.multiplicity = 1
                a0.wyckoff = "a"
                a0.uiso = "1.0"
                a0.element = cry.velement_name[i]
                cry.vatom_prim.append(a0)
                ii += 1
        cry.construct_structure_p1()
        cry.write_cif(self.matid+".relaxed.p1.cif")

        # cif (update)
        cry = structure.crystal.Crystal()
        cry.read_cif(self.cif)
        cry.check_cif()
        cry.construct_structure()
        #cry.read_structure_vasp("POSCAR"):
        cry.unit_prim[0] = numpy.array(self.cal.cell[0],float)
        cry.unit_prim[1] = numpy.array(self.cal.cell[1],float)
        cry.unit_prim[2] = numpy.array(self.cal.cell[2],float)
        for i,a in enumerate(self.cal.atom):
            cry.vatom_prim[i].position = numpy.array([a[1],a[2],a[3]],float)
        cry.reconstruct_structure()
        cry.write_cif(self.matid+".relaxed.cif")

        # cif (calc)
        #cry2 = cry.construct_structure_pcell()
        #cry2.write_cif(self.matid+".calc.cif")

    def kpoints(self):
        la,lb,lc = celllength(self.cal.cell)
        # ver 0.10
        kx = max(4,int(40/la))
        ky = max(4,int(40/lb))
        kz = max(4,int(40/lc))
        # ver 0.20
        rcell = cellrecip(self.cal.cell)
        ra,rb,rc = celllength(rcell)
        kx = max(4,int(30.0*ra+0.5))
        ky = max(4,int(30.0*rb+0.5))
        kz = max(4,int(30.0*rc+0.5))
        # kpoints_length
        if self.kpoints_length != None:
            kx = max(4,int(float(self.kpoints_length)*ra+0.5))
            ky = max(4,int(float(self.kpoints_length)*rb+0.5))
            kz = max(4,int(float(self.kpoints_length)*rc+0.5))
        # kpoints_spacing
        if self.kpoints_spacing != None:
            kx = max(4,int(ra/float(self.kpoints_spacing)+0.5))
            ky = max(4,int(rb/float(self.kpoints_spacing)+0.5))
            kz = max(4,int(rc/float(self.kpoints_spacing)+0.5))
        if(kx % 2 == 1): kx+=1
        if(ky % 2 == 1): ky+=1
        if(kz % 2 == 1): kz+=1
        # kpoint_max_mesh
        #print("kpoints_max_mesh:",self.kpoints_max_mesh)
        if self.kpoints_max_mesh != None:
            kx = min(kx,int(self.kpoints_max_mesh))
            ky = min(ky,int(self.kpoints_max_mesh))
            ky = min(ky,int(self.kpoints_max_mesh))
        #print("kpoints:",kx,ky,kz)
        if self.kpoints_mesh_nk1 > 0:
            kx = self.kpoints_mesh_nk1
        if self.kpoints_mesh_nk2 > 0:
            ky = self.kpoints_mesh_nk2
        if self.kpoints_mesh_nk1 > 0:
            kz = self.kpoints_mesh_nk3
        return [kx,ky,kz]

    def kpoints_band(self):
        cry = structure.crystal.Crystal()
        #cry.read_structure_vasp("POSCAR"):
        cry.unit_prim[0] = numpy.array(self.cal.cell[0],float)
        cry.unit_prim[1] = numpy.array(self.cal.cell[1],float)
        cry.unit_prim[2] = numpy.array(self.cal.cell[2],float)
        cry.velement_name = []
        for e in self.cal.ele:
            cry.velement_name.append(e)
        cry.velement_size = []
        for e in self.cal.natom_ele:
            cry.velement_size.append(int(e))
        a = self.cal.atom
        ii=0
        for i in range(len(cry.velement_name)):
            for j in range(cry.velement_size[i]):
                a0 = structure.atom.Atom()
                a0.position = numpy.array([a[ii][1],a[ii][2],a[ii][3]],float)
                a0.label = "%s%d" % (a[ii][0], j+1)
                a0.occupancy = 1.0
                a0.multiplicity = 1
                a0.wyckoff = "a"
                a0.uiso = "1.0"
                a0.element = cry.velement_name[i]
                cry.vatom_prim.append(a0)
                ii += 1
        cry.construct_structure_p1()
        kpath = cry.get_kpoints(int(self.kpoints_num_path))
        cry.write_brillouinzone_gnuplot(self.matid+".bz.plt")
        cry.write_brillouinzone_jmol(self.matid+".bz.spt")
        return kpath

    def ispin(self,mag):
        ispin = 1
        # spin_criterion 0.1
        #if(abs(float(mag)) > 0.1):
        if(abs(float(mag)) > float(self.spin_criterion)):
            ispin = 2
        #print("ispin:",ispin)
        return ispin

    def execute(self, task):
        print("execute...")
        f1 = task.option.get("_script_file", "job.sh")
        f2 = task.option.get("_log_file", "log")
        #cmd = "sh "+f1+" >& "+f2
        cmd = "bash "+f1+" >& "+f2
        os.system(cmd)

    def bandgaptype(self,f):
        doc = ElementTree.parse(f)
        root = doc.getroot()

        bandgap_type = None
        for e0 in root.findall("calculation"):
            if e0.attrib.get("name") == "dos" or e0.attrib.get("name") == "nscf":
                for e1 in e0.findall("properties/i"):
                    if e1.attrib.get("name") == "bandgap_type":
                        bandgap_type = e1.text
        return bandgap_type

    def kpoints_emass(self,f):
        #dk = 0.001
        dk = 0.005
        #dk = 0.01
        #dk = 0.05

        doc = ElementTree.parse(f)
        root = doc.getroot()

        primcell = []
        convcell = []
        ispin = None
        pcell = []
        kp = []  # kx,ky,kz
        kp_vb = []
        kp_cb = []
        for e0 in root.findall("structure"):
            if e0.attrib.get("name") == "original":
                for e1 in e0.findall("lattice"):
                    if e1.attrib.get("type") == "conventional":
                        for e2 in e1.findall("a1"):
                            l0 = e2.text.split()
                            convcell.append([l0[0],l0[1],l0[2]])
                        for e2 in e1.findall("a2"):
                            l0 = e2.text.split()
                            convcell.append([l0[0],l0[1],l0[2]])
                        for e2 in e1.findall("a3"):
                            l0 = e2.text.split()
                            convcell.append([l0[0],l0[1],l0[2]])
                    if e1.attrib.get("type") == "primitive":
                        for e2 in e1.findall("a1"):
                            l0 = e2.text.split()
                            primcell.append([l0[0],l0[1],l0[2]])
                        for e2 in e1.findall("a2"):
                            l0 = e2.text.split()
                            primcell.append([l0[0],l0[1],l0[2]])
                        for e2 in e1.findall("a3"):
                            l0 = e2.text.split()
                            primcell.append([l0[0],l0[1],l0[2]])
        for e0 in root.findall("calculation"):
            if e0.attrib.get("name") == "dos" or e0.attrib.get("name") == "nscf":
                for e1 in e0.findall("parameters/i"):
                    if e1.attrib.get("name") == "ispin":
                        ispin = int(e1.text)
                    for e1 in e0.findall("parameters/kpoints/i"):
                        l = e1.text.split()
                        kp.append([float(l[0]),float(l[1]),float(l[2])])
                for e1 in e0.findall("structure"):
                    if e1.attrib.get("name") == "final":
                        for e2 in e1.findall("lattice"):
                            if e2.attrib.get("type") == "primitive":
                                for e3 in e2.findall("a1"):
                                    l = e3.text.split()
                                    pcell.append([float(l[0]),float(l[1]),float(l[2])])
                                for e3 in e2.findall("a2"):
                                    l = e3.text.split()
                                    pcell.append([float(l[0]),float(l[1]),float(l[2])])
                                for e3 in e2.findall("a3"):
                                    l = e3.text.split()
                                    pcell.append([float(l[0]),float(l[1]),float(l[2])])
                for e1 in e0.findall("properties/i"):
                    if ispin == 1:
                        if e1.attrib.get("name") == "valence_band":
                            l = e1.text.split()
                            kp_vb.append([float(l[1]),float(l[2]),float(l[3])])
                        if e1.attrib.get("name") == "conduction_band":
                            l = e1.text.split()
                            kp_cb.append([float(l[1]),float(l[2]),float(l[3])])
                    if ispin == 2:
                        if e1.attrib.get("name") == "valence_band" and e1.attrib.get("spin") == "1":
                            l = e1.text.split()
                            kp_vb.append([float(l[1]),float(l[2]),float(l[3])])
                        if e1.attrib.get("name") == "conduction_band" and e1.attrib.get("spin") == "1":
                            l = e1.text.split()
                            kp_cb.append([float(l[1]),float(l[2]),float(l[3])])
                        if e1.attrib.get("name") == "valence_band" and e1.attrib.get("spin") == "2":
                            l = e1.text.split()
                            kp_vb.append([float(l[1]),float(l[2]),float(l[3])])
                        if e1.attrib.get("name") == "conduction_band" and e1.attrib.get("spin") == "2":
                            l = e1.text.split()
                            kp_cb.append([float(l[1]),float(l[2]),float(l[3])])

        primcell = numpy.array(primcell,float)
        convcell = numpy.array(convcell,float)
        prim2conv = numpy.dot(primcell,numpy.linalg.inv(convcell))
        conv2prim = numpy.dot(convcell,numpy.linalg.inv(primcell))
        #print(primcell)
        #print(convcell)

        #print("ispin",ispin)
        #print(pcell)
        print("kp_vb",kp_vb)
        print("kp_cb",kp_cb)

        pcell = numpy.array(pcell,float)
        rcell = cellrecip(pcell)
        print("primcell",pcell)
        print("recipcell",rcell)

        kpath = []

        if ispin == 1:
            # conv -> prim in recip space
            kp0 = numpy.dot(prim2conv.T,kp_vb[0])
            #kp0 = kp_vb[0]
            # recip -> cart
            #kp0 = numpy.dot(rcell,kp0)
            kp0 = numpy.dot(rcell.T,kp0)
            for i in range(3):
                for j in range(3):
                    for k in range(3):
                        kp1 = numpy.zeros((3))
                        kp1[0] = (i-1)*dk + kp0[0]
                        kp1[1] = (j-1)*dk + kp0[1]
                        kp1[2] = (k-1)*dk + kp0[2]
                        # cart -> recip
                        kp1 = numpy.dot(pcell,kp1)
                        kpath.append([kp1[0],kp1[1],kp1[2],1.0])
            kp0 = numpy.dot(prim2conv.T,kp_cb[0])
            #kp0 = kp_cb[0]
            #kp0 = numpy.dot(rcell,kp0)
            kp0 = numpy.dot(rcell.T,kp0)
            for i in range(3):
                for j in range(3):
                    for k in range(3):
                        kp1 = numpy.zeros((3))
                        kp1[0] = (i-1)*dk + kp0[0]
                        kp1[1] = (j-1)*dk + kp0[1]
                        kp1[2] = (k-1)*dk + kp0[2]
                        kp1 = numpy.dot(pcell,kp1)
                        kpath.append([kp1[0],kp1[1],kp1[2],1.0])

        elif ispin == 2:
            kp0 = numpy.dot(prim2conv.T,kp_vb[0])
            #kp0 = kp_vb[0]
            #kp0 = numpy.dot(rcell,kp0)
            kp0 = numpy.dot(rcell.T,kp0)
            for i in range(3):
                for j in range(3):
                    for k in range(3):
                        kp1 = numpy.zeros((3))
                        kp1[0] = (i-1)*dk + kp0[0]
                        kp1[1] = (j-1)*dk + kp0[1]
                        kp1[2] = (k-1)*dk + kp0[2]
                        kp1 = numpy.dot(pcell,kp1)
                        kpath.append([kp1[0],kp1[1],kp1[2],1.0])
            kp0 = numpy.dot(prim2conv.T,kp_cb[0])
            #kp0 = kp_cb[0]
            #kp0 = numpy.dot(rcell,kp0)
            kp0 = numpy.dot(rcell.T,kp0)
            for i in range(3):
                for j in range(3):
                    for k in range(3):
                        kp1 = numpy.zeros((3))
                        kp1[0] = (i-1)*dk + kp0[0]
                        kp1[1] = (j-1)*dk + kp0[1]
                        kp1[2] = (k-1)*dk + kp0[2]
                        kp1 = numpy.dot(pcell,kp1)
                        kpath.append([kp1[0],kp1[1],kp1[2],1.0])
            kp0 = numpy.dot(prim2conv.T,kp_vb[1])
            #kp0 = kp_vb[1]
            #kp0 = numpy.dot(rcell,kp0)
            kp0 = numpy.dot(rcell.T,kp0)
            for i in range(3):
                for j in range(3):
                    for k in range(3):
                        kp1 = numpy.zeros((3))
                        kp1[0] = (i-1)*dk + kp0[0]
                        kp1[1] = (j-1)*dk + kp0[1]
                        kp1[2] = (k-1)*dk + kp0[2]
                        kp1 = numpy.dot(pcell,kp1)
                        kpath.append([kp1[0],kp1[1],kp1[2],1.0])
            kp0 = numpy.dot(prim2conv.T,kp_cb[1])
            #kp0 = kp_cb[1]
            #kp0 = numpy.dot(rcell,kp0)
            kp0 = numpy.dot(rcell.T,kp0)
            for i in range(3):
                for j in range(3):
                    for k in range(3):
                        kp1 = numpy.zeros((3))
                        kp1[0] = (i-1)*dk + kp0[0]
                        kp1[1] = (j-1)*dk + kp0[1]
                        kp1[2] = (k-1)*dk + kp0[2]
                        kp1 = numpy.dot(pcell,kp1)
                        kpath.append([kp1[0],kp1[1],kp1[2],1.0])

        return kpath


class CalcStat():
    def __init__(self, program):
        self.program = program
        if self.program=="vasp":
            #self.cal = vasp.Vasp()
            self.cal = calculation.vasp.Vasp()
        elif self.program=="abinit":
            #self.cal = abinit.Abinit()
            self.cal = calculation.abinit.Abinit()
        elif self.program=="espresso":
            #self.cal = espresso.Espresso()
            self.cal = calculation.espresso.Espresso()

    def calstat(self):
        return self.cal.calstat()

def celllength(cell):
    a = cell[0]
    b = cell[1]
    c = cell[2]
    la = math.sqrt(numpy.dot(a,a))
    lb = math.sqrt(numpy.dot(b,b))
    lc = math.sqrt(numpy.dot(c,c))
    #print("cell_length:",la,lb,lc)
    return la,lb,lc

def cellangle(cell):
    a = cell[0]
    b = cell[1]
    c = cell[2]
    la,lb,lc = celllength(cell)
    #aa = math.acos(numpy.dot(b,c)/(lb*lc)) * 180.0 / math.pi
    #ab = math.acos(numpy.dot(c,a)/(lc*la)) * 180.0 / math.pi
    #ac = math.acos(numpy.dot(a,b)/(la*lc)) * 180.0 / math.pi
    aa = math.degrees(math.acos(numpy.dot(b,c)/(lb*lc)))
    ab = math.degrees(math.acos(numpy.dot(c,a)/(lc*la)))
    ac = math.degrees(math.acos(numpy.dot(a,b)/(la*lb)))
    #print("cell_angle:",aa,ab,ac)
    return aa,ab,ac

def cellrecip(cell):
    a = cell[0]
    b = cell[1]
    c = cell[2]
    rcell = numpy.zeros((3,3))
    v = cellvolume(cell)
    d = numpy.cross(b,c)
    rcell[0] = d / v
    d = numpy.cross(c,a)
    rcell[1] = d / v
    d = numpy.cross(a,b)
    rcell[2] = d / v
    #print("recip_cell:")
    #for i in xrange(3):
    #    print(rcell[i][0],rcell[i][1],rcell[i][2])
    return rcell

def cellvolume(cell):
    a = cell[0]
    b = cell[1]
    c = cell[2]
    d = numpy.cross(b,c)
    v = numpy.dot(a,d)
    #print("volume:",v)
    return v

def calc():
    arg = argparse.ArgumentParser(description="ac-calc")
    arg.add_argument("-p","--program",action="store",dest="program",default="vasp")
    arg.add_argument("-m","--mat",action="store",dest="mat")
    arg.add_argument("-n","--nproc",action="store",dest="nproc",default="0")
    arg.add_argument("-s","--scenario",action="store",dest="scenario",default="default")
    arg.add_argument("--all",action="store_true",dest="optsall")
    args = arg.parse_args()
    
    print()
    print("Template Oriented Atomic Simulation Toolkit (TOAST)")
    print("ver.", Calc.version)
    print()
    print("[auto claculation]")
    print("program:",args.program)
    print("mat:",args.mat)
    print("nproc:",args.nproc)

    cal = Calc(args.program, args.mat, args.nproc, args.scenario)
    # cal.program = args.program
    # cal.matid = args.mat
    # cal.nproc = args.nproc
    # cal.scenario = args.scenario

    fstat = open("STATUS_RUN", "w")
    start_time = time.time()
    fstat.write("START: {} {}\n".format(time.ctime(start_time), start_time))
    fstat.flush()
    try:
        cal.calculate()
    except:
        end_time = time.time()
        fstat.write("FAILED: {} {}\n".format(time.ctime(end_time), end_time))
        fstat.write("DURATION(sec.): {}\n".format(end_time - start_time))
        fstat.close()
        raise
    else:
        end_time = time.time()
        fstat.write("FINISH: {} {}\n".format(time.ctime(end_time), end_time))
        fstat.write("DURATION(sec.): {}\n".format(end_time - start_time))
        fstat.close()

if __name__ == "__main__":
    calc()
