#!/usr/bin/env python2
from __future__ import print_function

from collections import OrderedDict
import spglib
import os
import sys
import numpy as np
from math import *

_version = "0.0.4"

_cifdic = {"_symmetry_space_group_name_H-M": "string",
           "_symmetry_Int_Tables_number": "int",
           "_symmetry_space_group_name_Hall": "string",
           "_space_group_crystal_system": "string",
           "_cell_length_a": "float",
           "_cell_length_b": "float",
           "_cell_length_c": "float",
           "_cell_angle_alpha": "float",
           "_cell_angle_beta": "float",
           "_cell_angle_gamma": "float",
           "_symmetry_equiv_pos_as_xyz": "string",
           "_atom_site_label": "short_string",
           "_atom_site_fract_x": "float",
           "_atom_site_fract_y": "float",
           "_atom_site_fract_z": "float",
           "_atom_site_occupancy": "float",
           "_atom_site_symmetry_multiplicity": "short_int",
           "_atom_site_Wyckoff_symbol": "short_string",
           "_atom_site_type_symbol": "short_string",
           "_symmetry_equiv_pos_site_id": "int",
           "_symmetry_equiv_pos_as_xyz": "string"}

_cif_value_format = {"string":       "'{}'",
                     "int":          "{:d}",
                     "float":        "{:15.9f}",
                     "short_int":    "{:4d}",
                     "short_string": "{:4s}"}

def get_crystal_system(it_number):
    '''retrun crystal system from IT number'''
    if it_number <= 0:
        return "error"
    elif it_number < 3:
        return "triclinic"
    elif it_number < 16:
        return "monoclinic"
    elif it_number < 75:
        return "orthorhombic"
    elif it_number < 143:
        return "tetragonal"
    elif it_number < 168:
        return "trigonal"
    elif it_number < 195:
        return "hexagonal"
    elif it_number < 231:
        return "cubic"
    else:
        return "error"

def bravais_type(it_number):
    '''return lattice system from it_number'''
    pass


def write_cif_file(fname, cifdata):
    ''' write cif data to file
        fname: output filename
        cifdata:  cif data as a list of tuple (key, val)
                  key corresponds to a cif key.
                  if key is not loop_, print "key val"
                  if key is loop_, val[0] is a list of identifier in loop_ and val[1:] contains data.
    '''
    with open(fname, "w") as output:
        output.write("data_{}\n".format(os.path.splitext(os.path.basename(fname))[0]))
        for key,val in cifdata:
            if key != "loop_":
                keytype = _cifdic.get(key, "string")
                format_string = "{:36s} " + _cif_value_format[keytype] + "\n"
                output.write(format_string.format(key, val))

            else:
                output.write("loop_\n")
                loop_keys = val[0]
                loop_vals = val[1:]
                format_string = "   "
                for lk in loop_keys:
                    output.write("    {}\n".format(lk))
                    keytype = _cifdic.get(lk, "string")
                    format_string += " " + _cif_value_format[keytype]
                format_string += "\n"
                for lv in loop_vals:
                    output.write(format_string.format(*lv))


def celllength(cell):
    ''' calculate lattice constants from primitive vectors
        in
           cell[0], cell[1], cell[2]: primitive vectors
        return a,b,c
    '''
    a = cell[0]
    b = cell[1]
    c = cell[2]
    la = sqrt(np.dot(a,a))
    lb = sqrt(np.dot(b,b))
    lc = sqrt(np.dot(c,c))
    #print "cell_length:",la,lb,lc
    return la,lb,lc

def cellangle(cell):
    ''' calculate lattice angles (in degree) from primitive vectors
        in
           cell[0], cell[1], cell[2]: primitive vectors
        return alpha,beta,gamma
    '''
    a = cell[0]
    b = cell[1]
    c = cell[2]
    la,lb,lc = celllength(cell)
    aa = acos(np.dot(b,c)/(lb*lc)) * 180.0 / pi
    ab = acos(np.dot(c,a)/(lc*la)) * 180.0 / pi
    ac = acos(np.dot(a,b)/(la*lb)) * 180.0 / pi
    #print "cell_angle:",aa,ab,ac
    return aa,ab,ac


def read_poscar(filename):
    input = open(filename, "r")
    input.readline() # comment
    scale = float(input.readline())
    lattice = []
    for i in range(3):
        v = list(map(float, input.readline().strip().split()))
        lattice.append([v[0]*scale, v[1]*scale, v[2]*scale])
    atoms = input.readline().strip().split()
    numatoms = list(map(int, input.readline().strip().split()))
    numtot = sum(numatoms)

    numbers = []
    for i in range(len(numatoms)):
        for j in range(numatoms[i]):
            numbers.append(i)
    input.readline()  # Direct assumed
    positions  = []
    for i in range(numtot):
        v = list(map(float, input.readline().strip().split()))
        positions.append(v)
    input.close()

    return (lattice, positions, numbers), atoms

def print_cell(cell):
    lattice, positions, numbers = cell
    print("lattice vectors")
    for vec in lattice:
        print("{:15.9f} {:15.9f} {:15.9f}".format(vec[0], vec[1], vec[2]))
    print("atomic postions and type")
    for coor, atype in zip(positions, numbers):
        print("{:4d} {:15.9f} {:15.9f} {:15.9f}".format(atype, coor[0], coor[1], coor[2]))


def roundint(x):
    return int(floor(x+0.5))

def float2rational(x):
    '''make rational representation of x
       assume 0 < x < 1
       Denominator is assumed to be 2,3, or 4, 6.
    '''
    tol = 1.0e-3
    denoms = [2, 3, 4, 6]
    for d in denoms:
        w = x * d
        n = roundint(w)
        # print(x, d, w, n, w-n)
        if abs(w-n) < tol:
            s = "{:d}/{:d}".format(n, d)
            # print("float2rational:", x, s)
            return s
    sys.stderr.write("can't convert to rational {}\n".format(x))
    sys.exit(1)
        
    
def make_symop(rot, trans):
    '''make symop symbols from rotation matrix and translation vectors
       rot: rotational matrix in numpy 3x3 array
       trans: translational vector
       return string 
    '''
    symbols = []
    for i in range(3):
        symop = ""
        if rot[i,0] == 1:
            symop += "x"
        elif rot[i,0] == -1:
            symop += "-x"
        elif rot[i,0] == 0:
            pass
        else:
            print("error in rotation matrix")
        if rot[i,1] == 1:
            if symop:
                symop += "+y"
            else:
                symop += "y"
        elif rot[i,1] == -1:
            symop += "-y"
        elif rot[i,1] == 0:
            pass
        else:
            print("error in rotation matrix")
        if rot[i,2] == 1:
            if symop:
                symop += "+z"
            else:
                symop += "z"
        elif rot[i,2] == -1:
            symop += "-z"
        elif rot[i,2] == 0:
            pass
        else:
            print("error in rotation matrix")
        (shift, dummy) = modf(trans[i])
        if shift < 0:
            shift += 1.0
        if abs(shift) > 1.0e-4 and abs(shift-1.0) > 1.0e-4:
            if shift > 0:
                symop += "+"
            # symop += str(trans[i])
            symop += float2rational(trans[i])
        symbols.append(symop)
    return ",".join(symbols)
        

def sanitize_name_HM(name):
    work = []
    cur = ""
    length = len(name)
    i = 0
    while i < length:
        if name[i] not in "_/":
            if cur:
                work.append(cur)
            if name[i] == "-":
                cur = name[i:i+2]
                i += 2
            else:
                cur = name[i]
                i += 1
        elif name[i] == "_":
            cur += name[i+1]
            i += 2
        elif name[i] == "/":
            cur += name[i:i+2]
            i += 2
    if cur:
        work.append(cur)
    return " ".join(work)


def make_cifdata(cell, atoms, tol = 1.0e-5):
    cifdata = []

    scell = spglib.standardize_cell(cell)
    print("*** scell ***")
    print_cell(scell)
    dataset = spglib.get_symmetry_dataset(scell, tol)
    symmetry = spglib.get_symmetry(scell)
    
    wyckoffs = dataset["wyckoffs"]
    equiv_atoms = dataset["equivalent_atoms"]
    trans_matrix = dataset["transformation_matrix"]
    origin_shift = dataset["origin_shift"]
    hall_number = dataset["hall_number"]
    hall_symbol = dataset["hall"]
    sg_type = spglib.get_spacegroup_type(hall_number)
    print("trans_matrix")
    print(trans_matrix)
    print("origin_shift")
    print(origin_shift)

    lattice = scell[0]
    positions = scell[1]
    numbers = scell[2]
    # for n, pos in zip(numbers, positions):
    #     print(n, pos)
    
    
    # cifdata.append(("_symmetry_space_group_name_H-M", sg_type["international_full"]))
    name_HM = sanitize_name_HM(sg_type["international_short"])
    cifdata.append(("_symmetry_space_group_name_H-M", name_HM))
    cifdata.append(("_symmetry_Int_Tables_number", dataset["number"]))
    cifdata.append(("_symmetry_space_group_name_Hall", hall_symbol))
    crystal_system = get_crystal_system(dataset["number"])
    cifdata.append(("_space_group_crystal_system", crystal_system))

    a,b,c = celllength(lattice)
    alpha,beta,gamma = cellangle(lattice)
    cifdata.append(("_cell_length_a", a))
    cifdata.append(("_cell_length_b", b))
    cifdata.append(("_cell_length_c", c))
    cifdata.append(("_cell_angle_alpha", alpha))
    cifdata.append(("_cell_angle_beta", beta))
    cifdata.append(("_cell_angle_gamma", gamma))

    cifdata_symmetry_list = ("loop_", [])
    cifdata_symmetry_list[1].append(["_symmetry_equiv_pos_site_id", "_symmetry_equiv_pos_as_xyz"])

    ind = 0
    for rot,trans in zip(dataset['rotations'], dataset['translations']):
        symbols = make_symop(rot, trans)
        ind += 1
        cifdata_symmetry_list[1].append([ind, symbols])
        # print("rotation")
        # print(rot)
        # print("translation")
        # print(trans)
        # print(symbols)

    cifdata.append(cifdata_symmetry_list)

    # find inequivalent atoms and calculate their multiplicitis
    inequiv_atoms = OrderedDict()
    for num in equiv_atoms:
        if num not in inequiv_atoms:
            inequiv_atoms[num] = 1
        else:
            inequiv_atoms[num] += 1

    sg = dataset["international"].strip()

    print("space group number {}".format(dataset["number"]))
    print("space group {}".format(dataset["international"]))
    print("hall symbol {}".format(dataset["hall"]))
          
    print("name wyckoff  multiplicity  x y z")

    cifdata_atomlist = ("loop_", [])
    cifdata_atomlist[1].append(["_atom_site_label",  "_atom_site_fract_x", "_atom_site_fract_y", "_atom_site_fract_z",
                                "_atom_site_occupancy", "_atom_site_symmetry_multiplicity", "_atom_site_Wyckoff_symbol",
                                "_atom_site_type_symbol"])
    suffix_list = {}
    for atom in atoms:
        suffix_list[atom] = 1
    for num in inequiv_atoms:
        atom_type_number = numbers[num]
        atom_type = atoms[atom_type_number]
        atom_label = atom_type + str(suffix_list[atom_type])
        suffix_list[atom_type] += 1
        cifdata_atomlist[1].append([atom_label, positions[num][0], positions[num][1], positions[num][2],
                                    1.0, inequiv_atoms[num], wyckoffs[num], atom_type])
        print(atom_label, atom_type, wyckoffs[num], inequiv_atoms[num], positions[num][0], positions[num][1], positions[num][2])
    cifdata.append(cifdata_atomlist)
    return cifdata


if __name__ == "__main__":
    import sys
    import argparse

    parser = argparse.ArgumentParser(description="convert POSCAR to CIF")
    parser.add_argument("files", nargs="+", metavar="file", type=str, help = "CIF flename")
    parser.add_argument("-t", "--tol", metavar="diff", type=float, default=None, help="precision in spglib")
    
    args = parser.parse_args()

    print("poscar2cif version: ", _version)

    tol = 1.0e-5
    if args.tol:
        tol = args.tol
    for f in args.files:
        cell, atoms = read_poscar(f)
        cifdata = make_cifdata(cell, atoms, tol)
        write_cif_file(f+".cif", cifdata)
    
    # for f in glob.glob("*.POSCAR"):
    #     print("filename = {}".format(f))
    #     cell, atoms = read_poscar(f)
    #     pcell = spglib.find_primitive(cell)
    #     print("*** pcell ***")
    #     print(pcell)
    #     print_cell(cell)

    #     cifdata = make_cifdata(cell, atoms)
    #     write_cif_file(f+".cif", cifdata)
    #     # find_symmetry(pcell, atoms)
    #     # scell = spglib.standardize_cell(cell)
    #     # print(scell)
